
#ifndef __KERNEL__
#define __KERNEL__
#endif

#ifndef MODULE
#define MODULE
#endif
#define __DRIVER__
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/malloc.h>
#include <linux/fs.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/io.h>

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
 *  Function prototypes.
 */
static int device_open(struct inode *, struct file *); 
static int device_close(struct inode *, struct file *); 
int __init init_module(void);
void cleanup_module(void);

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

static const char dev[]="restart";
#define MAJ 188

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/* 
 *   File operation structure contains basic functions to operate a device
 */
static struct file_operations device_fops = {
#if defined(THIS_MODULE)
    THIS_MODULE,
#endif 
    NULL,                             /* llseek      */
    NULL,                             /* read        */
    NULL,                             /* write       */
    NULL,                             /* readdir     */
    NULL,                             /* poll        */
    NULL,                             /* ioctl       */
    NULL,                             /* mmap        */
    device_open,                      /* open        */
    NULL,                             /* flush       */
    device_close,                     /* close       */
    0, };

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
 *  Open the device.
 */
static int device_open(struct inode *inp, struct file *fp)
{
    __asm__ __volatile__ ( "movl $1, %eax\n movl %eax, %cr0\n");
    return 0;  /* Will never happen */
} 
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
 *  Close the device.
 */
static int device_close(struct inode *inp, struct file *fp)
{
    return 0;
} 
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
 *   Initialize and register the module
 */
int __init init_module()
{
    int result;
    if((result = register_chrdev(MAJ, dev, &device_fops)) < 0)
    {
        printk(KERN_ALERT "Can't register %s by major number %d\n", dev, MAJ);
        return result;
    }
    return 0;
}
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
 *   Release the module
 */
void cleanup_module()
{
    int result;

    if((result = unregister_chrdev(MAJ, dev)) < 0)
    {
        printk(KERN_ALERT "Can't unregister \"%s\" (%d)\n", dev, result);
        return;
    }
}
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
